@@ -98,6 +98,9 @@ def assert_int(x):
     return type(x) in [int, np.int, np.int8, np.int16, np.uint8,
                        np.uint16, np.int32, np.int64, np.uint32, np.uint64]
 
+def assert_iterable(x):
+    return hasattr(x, '__iter__')
+
 def npa(arr=[], **kwargs):
     '''Returns np.array of some kind.
     Optional params:
@@ -105,16 +108,16 @@ def npa(arr=[], **kwargs):
         - ones: tuple. If provided, returns np.ones(ones)
         - empty: tuple. If provided, returns np.empty(empty)
         - dtype: numpy datatype. If provided, returns np.array(arr, dtype=dtype) .'''
+
+    dtype=kwargs['dtype'] if 'dtype' in kwargs.keys() else None
     if 'zeros' in kwargs.keys():
-        arr = np.zeros(kwargs['zeros'])
+        arr = np.zeros(kwargs['zeros'], dtype=dtype)
     elif 'ones' in kwargs.keys():
-        arr = np.ones(kwargs['ones'])
+        arr = np.ones(kwargs['ones'], dtype=dtype)
     elif 'empty' in kwargs.keys():
-        arr = np.empty(kwargs['empty'])
+        arr = np.empty(kwargs['empty'], dtype=dtype)
     else:
-        arr=np.array(arr)
-    if 'dtype' in kwargs.keys():
-        arr = np.array(arr, dtype=kwargs['dtype'])
+        arr=np.array(arr, dtype=dtype)
     return arr
 
 def isnumeric(x):
@@ -223,7 +226,7 @@ def thresh(arr, th, sgn=1, pos=1):
     - pos: 1 or -1, position of closest value, just following or just preceeding.'''
     assert pos in [-1,1]
     assert sgn in [-1,1]
-    arr=np.asarray(arr)
+    arr=np.asarray(arr).copy()
     assert arr.ndim==1
     arr= (arr-th)*sgn+th # Flips the array around threshold if sgn==-1
 
@@ -233,6 +236,19 @@ def thresh(arr, th, sgn=1, pos=1):
         return np.array([])
     return  i[arr[np.clip(i-1, 0, len(arr)-1)]<th] if pos==1 else i[arr[np.clip(i+1, 0, len(arr)-1)]>th]
 
+def thresh_fast(arr, th, sgn=1, pos=1):
+    '''Returns indices of the data points closest to a directed crossing of th.
+    - data: numpy array.
+    - th: threshold value.
+    - sgn: 1 or -1, direction of the threshold crossing, either positive (1) or negative (-1).
+    - pos: 1 or -1, position of closest value, just following or just preceeding.'''
+    assert pos in [-1,1]
+    assert sgn in [-1,1]
+    assert arr.ndim==1
+    m=(arr>th).astype(np.int8)
+    ths=np.nonzero(np.diff(m)==sgn)[0]
+    if pos: ths+=1
+    return ths
 
 def thresh_consec(arr, th, sgn=1, n_consec=0, exclude_edges=True, only_max=False, ret_values=True):
     '''
@@ -387,6 +403,11 @@ def get_bins(cwin, cbin):
     else: # odd
         return np.arange(-cwin/2+cbin/2, cwin/2+cbin/2, cbin)
 
+def find_nearest(array, value):
+    array = np.asarray(array)
+    idx = (np.abs(array - value)).argmin()
+    return idx
+
 def mask_2d(x, m):
     '''
     Mask a 2D array and preserve the
@@ -426,7 +447,7 @@ def make_2D_array(arr_lis, accept_heterogeneous=False):
     return arr
 
 @njit(cache=True)
-def split(arr, sample_size=0, n_samples=0, overlap=0, return_last=True, prnt=True):
+def split(arr, sample_size=0, n_samples=0, overlap=0, return_last=True, verbose=True):
     '''
     Parameters:
         - arr: array to split into EITHER n_samples OR samples of size sample_size.
@@ -457,7 +478,7 @@ def split(arr, sample_size=0, n_samples=0, overlap=0, return_last=True, prnt=Tru
     s=sample_size
     step = s-round(s*overlap)
     real_o=round((s-step)/s, 2)
-    if overlap!=real_o and prnt: print('Real overlap: ', round(real_o, 2))
+    if overlap!=real_o and verbose: print('Real overlap: ', round(real_o, 2))
     samples = List([arr[i : i + s] for i in range(0, len(arr), step)])
 
     # always return last sample if len matches
@@ -543,8 +564,7 @@ def align_timeseries(timeseries, sync_signals, fs, offset_policy='original'):
 
     Nevents, totDft, avDft, stdDft = len(sync_signals[0]), (sync_signals[1]-sync_signals[0])[-1], np.mean(np.diff(sync_signals[1]-sync_signals[0])), np.std(np.diff(sync_signals[1]-sync_signals[0]))
     totDft, avDft, stdDft = totDft*1000/fs_master, avDft*1000/fs_master, stdDft*1000/fs_master
-    print("{} sync events used for alignement - start-end drift of {}ms, \
-          av. drift between consec. sync events of {}+/-{}ms.".format(Nevents, round(totDft,4), round(avDft,4), round(stdDft,4)))
+    print("{} sync events used for alignement - start-end drift of {}ms".format(Nevents, round(totDft,3)))
 
     for dataset_i in range(len(timeseries)):
         if dataset_i==0: continue #first dataset is reference so left untouched
@@ -561,6 +581,49 @@ def align_timeseries(timeseries, sync_signals, fs, offset_policy='original'):
 
     return timeseries if usage==1 else timeseries[1]
 
+
+def align_timeseries_interpol(timeseries, sync_signals, fs=None):
+    '''
+    Align a list of N timeseries in the frame of reference of the first timeserie.
+
+    Assumes that the drift is going to be linear, it is interpolated for times far from sync signals
+    (sync_signal1 = a * sync_signal0 + b).
+
+    timeseries: list of np arrays (len N), timeseries to align. In samples to ensure accurate alignment
+    sync_signals: list of np arrays (len N), sync signals respective to timeseries. In samples to ensure accurate alignment
+    fs: float or list of floats (len N), sampling frequencies of time series.
+        fs is optional (only to get drift and offset in seconds).
+
+    returns:
+        - timeseries: list of np arrays (len N, in samples), timeries aligned in temporal reference frame of timeseries[0]
+    '''
+
+    # Parameters formatting and checks
+    assert len(timeseries)>=2
+    assert len(sync_signals)==len(timeseries)
+    if fs is not None:
+        if assert_iterable(fs):assert len(timeseries)==len(fs)
+        else:fs=[fs]*len(timeseries)
+    for tsi, ts in enumerate(timeseries):
+        assert np.all(ts.astype(int)==ts), 'Timeseries need to be integers, in samples acquired at fs sampling rate!'
+        timeseries[tsi]=ts.astype(int)
+    for tsi, ts in enumerate(sync_signals):
+        assert np.all(ts.astype(int)==ts), 'Sync signals need to be integers, in samples acquired at fs sampling rate!'
+        sync_signals[tsi]=ts.astype(int)
+
+    # Align
+    master_sync=sync_signals[0]
+
+    for i, (ts, ss) in enumerate(zip(timeseries[1:], sync_signals[1:])):
+        (a, b) = np.polyfit(ss, master_sync, 1)
+        if fs is not None:
+            drift=round(abs(a*fs[i]/fs[0]-1)*3600*1000,2)
+            offset=round(b/fs[0],2)
+            print(f'Drift (assumed linear) of {drift}ms/h, \noffset of {offset}s between ephys and paq files.\n')
+        timeseries[i]=(a*ts+b).astype(int)
+
+    return timeseries
+
 #%% Stolen from phy
 def _as_array(arr, dtype=None):
     """Convert an object to a numerical NumPy array.
@@ -1480,64 +1543,4 @@ def zero_crossings_sine_fit(y_axis, x_axis, fit_window = None, smooth_window = 1
         true_crossings.append(popt[0])
 
 
-    return true_crossings
-
-
-
-
-def _test_zero():
-    _max, _min = peakdetect_zero_crossing(y,x)
-def _test():
-    _max, _min = peakdetect(y,x, delta=0.30)
-
-
-def _test_graph():
-    i = 10000
-    x = np.linspace(0,3.7*pi,i)
-    y = (0.3*np.sin(x) + np.sin(1.3 * x) + 0.9 * np.sin(4.2 * x) + 0.06 *
-    np.random.randn(i))
-    y *= -1
-    x = range(i)
-
-    _max, _min = peakdetect(y,x,750, 0.30)
-    xm = [p[0] for p in _max]
-    ym = [p[1] for p in _max]
-    xn = [p[0] for p in _min]
-    yn = [p[1] for p in _min]
-
-    plot = pylab.plot(x,y)
-    pylab.hold(True)
-    pylab.plot(xm, ym, "r+")
-    pylab.plot(xn, yn, "g+")
-
-    _max, _min = peak_det_bad.peakdetect(y, 0.7, x)
-    xm = [p[0] for p in _max]
-    ym = [p[1] for p in _max]
-    xn = [p[0] for p in _min]
-    yn = [p[1] for p in _min]
-    pylab.plot(xm, ym, "y*")
-    pylab.plot(xn, yn, "k*")
-    pylab.show()
-
-def _test_graph_cross(window = 11):
-    i = 10000
-    x = np.linspace(0,8.7*pi,i)
-    y = (2*np.sin(x) + 0.006 *
-    np.random.randn(i))
-    y *= -1
-    pylab.plot(x,y)
-    #pylab.show()
-
-
-    crossings = zero_crossings_sine_fit(y,x, smooth_window = window)
-    y_cross = [0] * len(crossings)
-
-
-    plot = pylab.plot(x,y)
-    pylab.hold(True)
-    pylab.plot(crossings, y_cross, "b+")
-    pylab.show()
-
-def _is_array_like(arr):
-    return isinstance(arr, (list, np.ndarray))
-
+    return true_crossings
\ No newline at end of file
